package migl.lisp.operator;

import java.util.Map;
import java.util.HashMap;

import migl.lisp.LispElement;
import migl.lisp.LispError;
import migl.lisp.LispImpl;
import migl.lisp.LispOperator;
import migl.util.ConsList;

public class DefineOperator implements LispOperator {
	
	private static Map<String, ConsList<Object>> define = new HashMap<>();
	
	/**
	 * Vide la liste des variable et expression lambda definit
	 */
	public static void clear() {
		define.clear();
	}
	
	@SuppressWarnings("unchecked")
	public static LispElement eval(Object elt) throws LispError {
		if(elt instanceof ConsList) {
			return eval((ConsList<Object>) elt);
		}
		//Recup la valeur de l'element
		LispElement lelt = LispElement.generate(elt);
		String key;
		try {
			key = lelt.toStr();
		} catch(IllegalStateException ex) {
			throw new LispError(lelt.toString() + LispError.ERR_UNKNOW);
		}
		//Recup la valeur dans la map
		ConsList<Object> cl = define.get(key);
		if(cl == null) {
			//Si aucune valeur alors erreur
			throw new LispError(key + LispError.ERR_UNKNOW);
		}
		//ToDo verifier si ce n'est pas une lambda expression
		return LispElement.generate(cl.car());
	}
	
	public static LispElement eval(ConsList<Object> lisp) throws LispError {
		if(lisp.car() instanceof ConsList) {
			String res = lisp.car().toString();
			return LispElement.generate(res.substring(1, res.length() - 1));
		}
		return LispElement.generate(lisp.car());
	}
	
	private static LispElement evalLambda(ConsList<Object> lisp) {
		return null;
	}

	@Override
	public LispElement apply(String operator, ConsList<Object> lisp) throws LispError {
		if(lisp.size() != 2) {
			throw new LispError(LispError.ERR_NUM_ARG);
		}
		switch(operator) {
			case "define":
				return this.define(lisp);
			case "set!":
				return this.set(lisp);
			default:
				throw new LispError(operator + LispError.ERR_UNKNOW);
		}
	}
	
	private LispElement define(ConsList<Object> lisp) throws LispError {
		//Recup la clef
		LispElement le = LispElement.generate(lisp.car());
		String key;
		try {
			key = le.toStr();
		} catch(IllegalStateException ex) {
			throw new LispError(le.toString() + LispError.ERR_INVALID, ex);
		}
		//Verification de sa validité
		LispImpl.verifyForbiddenName(key);
		//Ajoute dans la map
		define.put(key, lisp.cdr());
		//Evalue pour le retour
		return eval(lisp.cdr());
	}

	private LispElement set(ConsList<Object> lisp) throws LispError {
		//Recup la clef
		String key;
		try {
			key = LispElement.generate(lisp.car()).toStr();
		} catch(IllegalStateException ex) {
			throw new LispError(ex);
		}
		//Verification de sa validité
		LispImpl.verifyForbiddenName(key);
		//Regarde si la valeur est deja presente
		if(!define.containsKey(key)) {
			throw new LispError(key + LispError.ERR_UNKNOW);
		}
		//Ajoute dans la map
		define.put(key, lisp.cdr());
		//Evalue pour le retour
		return eval(lisp.cdr());
	}

}
